Package: tibble
Title: Simple Data Frames
Version: 3.1.7
Authors@R: 
    c(person(given = "Kirill",
             family = "M\u00fcller",
             role = c("aut", "cre"),
             email = "krlmlr+r@mailbox.org"),
      person(given = "Hadley",
             family = "Wickham",
             role = "aut",
             email = "hadley@rstudio.com"),
      person(given = "Romain",
             family = "Francois",
             role = "ctb",
             email = "romain@r-enthusiasts.com"),
      person(given = "Jennifer",
             family = "Bryan",
             role = "ctb",
             email = "jenny@rstudio.com"),
      person(given = "RStudio",
             role = c("cph", "fnd")))
Description: Provides a 'tbl_df' class (the 'tibble') with stricter checking and better formatting than the traditional
    data frame.
License: MIT + file LICENSE
URL: https://tibble.tidyverse.org/, https://github.com/tidyverse/tibble
BugReports: https://github.com/tidyverse/tibble/issues
Depends: R (>= 3.1.0)
Imports: ellipsis (>= 0.3.2), fansi (>= 0.4.0), lifecycle (>= 1.0.0),
        magrittr, methods, pillar (>= 1.7.0), pkgconfig, rlang (>=
        1.0.1), utils, vctrs (>= 0.3.8)
Suggests: bench, bit64, blob, brio, callr, cli, covr, crayon (>=
        1.3.4), DiagrammeR, dplyr, evaluate, formattable, ggplot2, hms,
        htmltools, knitr, lubridate, mockr, nycflights13, pkgbuild,
        pkgload, purrr, rmarkdown, stringi, testthat (>= 3.0.2), tidyr,
        withr
VignetteBuilder: knitr
Encoding: UTF-8
RoxygenNote: 7.1.2
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: subsetting, as_tibble, add, invariants
Config/autostyle/scope: line_breaks
Config/autostyle/strict: true
Config/Needs/website: tidyverse/tidytemplate
NeedsCompilation: yes
Packaged: 2022-04-26 14:15:54 UTC; kirill
Author: Kirill Müller [aut, cre],
  Hadley Wickham [aut],
  Romain Francois [ctb],
  Jennifer Bryan [ctb],
  RStudio [cph, fnd]
Maintainer: Kirill Müller <krlmlr+r@mailbox.org>
Repository: CRAN
Date/Publication: 2022-05-03 07:00:06 UTC
