Package: tibble
Title: Simple Data Frames
Version: 3.3.1
Authors@R: c(
    person("Kirill", "Müller", , "kirill@cynkra.com", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0002-1416-3412")),
    person("Hadley", "Wickham", , "hadley@rstudio.com", role = "aut"),
    person("Romain", "Francois", , "romain@r-enthusiasts.com", role = "ctb"),
    person("Jennifer", "Bryan", , "jenny@rstudio.com", role = "ctb"),
    person("Posit Software, PBC", role = c("cph", "fnd"),
           comment = c(ROR = "03wc8by49"))
  )
Description: Provides a 'tbl_df' class (the 'tibble') with stricter
    checking and better formatting than the traditional data frame.
License: MIT + file LICENSE
URL: https://tibble.tidyverse.org/, https://github.com/tidyverse/tibble
BugReports: https://github.com/tidyverse/tibble/issues
Depends: R (>= 3.4.0)
Imports: cli, lifecycle (>= 1.0.0), magrittr, methods, pillar (>=
        1.8.1), pkgconfig, rlang (>= 1.0.2), utils, vctrs (>= 0.5.0)
Suggests: bench, bit64, blob, brio, callr, DiagrammeR, dplyr, evaluate,
        formattable, ggplot2, here, hms, htmltools, knitr, lubridate,
        nycflights13, pkgload, purrr, rmarkdown, stringi, testthat (>=
        3.0.2), tidyr, withr
VignetteBuilder: knitr
Config/autostyle/rmd: false
Config/autostyle/scope: line_breaks
Config/autostyle/strict: true
Config/Needs/website: tidyverse/tidytemplate
Config/testthat/edition: 3
Config/testthat/parallel: true
Config/testthat/start-first: vignette-formats, as_tibble, add,
        invariants
Config/usethis/last-upkeep: 2025-06-07
Encoding: UTF-8
RoxygenNote: 7.3.3.9000
NeedsCompilation: yes
Packaged: 2026-01-10 18:28:47 UTC; kirill
Author: Kirill Müller [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1416-3412>),
  Hadley Wickham [aut],
  Romain Francois [ctb],
  Jennifer Bryan [ctb],
  Posit Software, PBC [cph, fnd] (ROR: <https://ror.org/03wc8by49>)
Maintainer: Kirill Müller <kirill@cynkra.com>
Repository: CRAN
Date/Publication: 2026-01-11 10:10:02 UTC
