% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/deprecated.R
\name{name-repair}
\alias{name-repair}
\title{Name repair}
\description{
Please review \code{\link[vctrs:vec_as_names]{vctrs::vec_as_names()}}.
}
\keyword{internal}
